# Copyright 2015 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import tables

from cloudkittydashboard.api import cloudkitty as api

ENABLE = 0
DISABLE = 1


class EditModulePriority(tables.LinkAction):
    name = "editmodulepriority"
    verbose_name = _("Edit module priority")
    icon = "edit"
    ajax = True
    classes = ("ajax-modal",)

    def get_link_url(self, datum):
        if datum.module_id:
            url = "horizon:admin:rating_modules:edit_priority"
            return reverse(url, kwargs={'module_id': datum.module_id})


class ToggleEnabledModule(tables.BatchAction):
    name = "toggle_module"
    data_type_singular = _("Module")
    data_type_plural = _("Modules")
    classes = ("btn-toggle",)

    @staticmethod
    def action_present(count):
        return (
            ngettext_lazy(
                u"Enable Module",
                u"Enable Modules",
                count
            ),
            ngettext_lazy(
                u"Disable Module",
                u"Disable Modules",
                count
            ),
        )

    @staticmethod
    def action_past(count):
        return (
            ngettext_lazy(
                u"Enabled Module",
                u"Enabled Modules",
                count
            ),
            ngettext_lazy(
                u"Disabled Module",
                u"Disabled Modules",
                count
            ),
        )

    def allowed(self, request, module=None):
        self.enabled = module.get('enabled')
        if self.enabled:
            self.current_present_action = DISABLE
        else:
            self.current_present_action = ENABLE
        return True

    def action(self, request, obj_id):
        client = api.cloudkittyclient(request)
        module = client.rating.get_module(module_id=obj_id)
        self.enabled = module.get('enabled', False)
        self.current_past_action = DISABLE if self.enabled else ENABLE
        client.rating.update_module(module_id=obj_id,
                                    enabled=(not self.enabled))


def get_details_link(datum):
    if datum.module_id:
        url = "horizon:admin:rating_modules:module_details"
        return reverse(url, kwargs={'module_id': datum.module_id})


class ModulesTable(tables.DataTable):
    name = tables.Column('name', verbose_name=_("Name"), link=get_details_link)
    description = tables.Column('description', verbose_name=_("Description"))
    hot_config = tables.Column('hot-config', verbose_name=_("Configurable"))
    priority = tables.Column('priority', verbose_name=_("Priority"))
    enabled = tables.Column('enabled', verbose_name=_("Enabled"))

    class Meta(object):
        name = "modules"
        verbose_name = _("Modules")
        row_actions = (ToggleEnabledModule, EditModulePriority)



# Copyright 2015 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from horizon import forms
from horizon import tables
from horizon import views

from cloudkittydashboard.api import cloudkitty as api
from cloudkittydashboard.dashboards.admin.pyscripts import forms \
    as pyscripts_forms
from cloudkittydashboard.dashboards.admin.pyscripts import tables \
    as pyscripts_tables


class IndexView(tables.DataTableView):
    table_class = pyscripts_tables.PyScriptsTable
    template_name = 'admin/pyscripts/pyscripts_list.html'

    def get_data(self):
        data = api.cloudkittyclient(
            self.request).rating.pyscripts.list_scripts()['scripts']
        data = api.identify(data, key='script_id')
        return data


class ScriptCreateView(forms.ModalFormView):
    form_class = pyscripts_forms.CreateScriptForm
    form_id = "create_script"
    modal_header = _("Create Script")
    page_title = _("Create Script")
    submit_url = reverse_lazy('horizon:admin:pyscripts:script_create')
    success_url = reverse_lazy('horizon:admin:pyscripts:index')
    template_name = 'admin/pyscripts/form.html'

    def get_object_id(self, obj):
        return obj


class ScriptUpdateView(forms.ModalFormView):
    form_class = pyscripts_forms.EditScriptForm
    form_id = "update_script"
    modal_header = _("Update Script")
    page_title = _("Update Script")
    submit_url = 'horizon:admin:pyscripts:script_update'
    success_url = 'horizon:admin:pyscripts:script_update'
    template_name = 'admin/pyscripts/form.html'

    def get_initial(self):
        client = api.cloudkittyclient(self.request)

        script = client.rating.pyscripts.get_script(
            script_id=self.kwargs['script_id'])
        self.initial = script
        self.initial['script_data'] = self.initial['data']
        return self.initial

    def get_context_data(self, **kwargs):
        context = super(ScriptUpdateView, self).get_context_data(**kwargs)
        context['script_id'] = self.kwargs.get('script_id')
        context['submit_url'] = reverse_lazy(self.submit_url,
                                             args=(context['script_id'], ))
        return context

    def get_success_url(self, **kwargs):
        return reverse('horizon:admin:pyscripts:index')


class ScriptDetailsView(views.APIView):
    template_name = 'admin/pyscripts/details.html'
    page_title = _("Script Details : {{ script.name }}")

    def get_data(self, request, context, *args, **kwargs):
        script_id = kwargs.get("script_id")
        try:
            client = api.cloudkittyclient(self.request)
            script = client.rating.pyscripts.get_script(script_id=script_id)
        except Exception:
            script = None
        context['script'] = script
        return context



# -*- coding: utf-8 -*-
# Copyright 2015 Objectif Libre
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from collections import abc

from django.conf import settings
from horizon.utils.memoized import memoized  # noqa
from keystoneauth1.identity.v3 import Token

from cloudkittyclient import client as ck_client
from cloudkittydashboard import utils


@memoized
def cloudkittyclient(request, version='1'):
    """Initialization of Cloudkitty client."""
    cacert = getattr(settings, 'OPENSTACK_SSL_CACERT', None)
    insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
    auth_url = getattr(settings, 'OPENSTACK_KEYSTONE_URL', None)
    interface = getattr(settings, 'OPENSTACK_ENDPOINT_TYPE', 'publicURL')
    auth = Token(
        auth_url,
        token=request.user.token.id,
        project_id=request.user.project_id,
        domain_id=request.user.domain_id,
    )

    adapter_options = {
        'region_name': request.user.services_region,
        'interface': interface,
    }

    return ck_client.Client(
        version,
        auth=auth,
        cacert=cacert,
        insecure=insecure,
        adapter_options=adapter_options,
    )


def identify(what, name=False, key=None):
    if isinstance(what, abc.Iterable):
        for i in what:
            i['id'] = i.get(key or "%s_id" % i['key'])
            if name and not i.get('name'):
                i['name'] = i.get(key or "%s_id" % i['key'])
        what = [utils.TemplatizableDict(i) for i in what]
    else:
        what['id'] = what.get(key or "%s_id" % what['key'])
        if name and not i.get('name'):
            what['name'] = what.get(key or "%s_id" % what['key'])
        what = utils.TemplatizableDict(what)
    return what



# Copyright 2015 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from collections import OrderedDict
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy

from horizon import tables
from horizon import tabs

from cloudkittydashboard.api import cloudkitty as api


class CreateService(tables.LinkAction):
    name = "createservice"
    verbose_name = _("Create new Service")
    url = 'horizon:admin:hashmap:service_create'
    icon = "plus"
    ajax = True
    classes = ("ajax-modal",)


class DeleteService(tables.DeleteAction):
    name = "deleteservice"
    verbose_name = _("Delete Service")
    data_type_singular = _("Service")
    data_type_plural = _("Services")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Delete Service",
            u"Delete Services",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Deleted Service",
            u"Deleted Services",
            count
        )

    def action(self, request, service_id):
        api.cloudkittyclient(request).rating.hashmap.delete_service(
            service_id=service_id)


class ServicesTable(tables.DataTable):
    """This table list the available services.

    Clicking on a service name sends you to a ServiceTabs page.
    """
    name = tables.Column('name', verbose_name=_("Name"),
                         link='horizon:admin:hashmap:service')
    unit = tables.Column("unit", verbose_name=_("Unit"))

    def get_object_id(self, datum):
        return datum['id']

    def get_object_display(self, datum):
        return datum['name']

    class Meta(object):
        name = "services"
        verbose_name = _("Services")
        table_actions = (CreateService, DeleteService)
        row_actions = (DeleteService,)


class CreateGroup(tables.LinkAction):
    name = "creategroup"
    verbose_name = _("Create new Group")
    icon = "plus"
    ajax = True
    classes = ("ajax-modal",)

    def get_link_url(self, datum=None):
        url = 'horizon:admin:hashmap:group_create'
        service_id = self.table.request.service_id
        return reverse(url, args=[service_id])


class DeleteGroup(tables.DeleteAction):
    name = "deletegroup"
    verbose_name = _("Delete Group")
    data_type_singular = _("Group")
    data_type_plural = _("Groups")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Delete Group",
            u"Delete Groups",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Deleted Group",
            u"Deleted Groups",
            count
        )

    def action(self, request, group_id):
        api.cloudkittyclient(request).rating.hashmap.delete_group(
            group_id=group_id)


def get_detail_link(datum):
    if datum.group_id:
        url = "horizon:admin:hashmap:group_details"
        return reverse(url, kwargs={'group_id': datum.group_id})


class GroupsTable(tables.DataTable):
    """This table list the available groups.

    Clicking on a group name sends you to a GroupsTab page.
    """
    name = tables.Column('name', verbose_name=_("Name"), link=get_detail_link)
    group_id = tables.Column('group_id', verbose_name=_("Group"))

    class Meta(object):
        name = "groups"
        verbose_name = _("Groups")
        table_actions = (CreateGroup, DeleteGroup)
        row_actions = (DeleteGroup,)


class GroupsTab(tabs.TableTab):
    name = _("Groups")
    slug = "hashmap_groups"
    table_classes = (GroupsTable,)
    template_name = "horizon/common/_detail_table.html"
    preload = True

    def get_groups_data(self):
        client = api.cloudkittyclient(self.request)
        groups = client.rating.hashmap.get_group().get('groups', [])
        return api.identify(groups, key='group_id')


class CreateServiceThreshold(tables.LinkAction):
    name = "createservicethreshold"
    verbose_name = _("Create new Service Threshold")
    icon = "plus"
    ajax = False
    classes = ("ajax-modal",)

    def get_link_url(self, datum=None):
        url = 'horizon:admin:hashmap:service_threshold_create'
        service_id = self.table.request.service_id
        return reverse(url, args=[service_id])


class CreateFieldThreshold(tables.LinkAction):
    name = "createfieldthreshold"
    verbose_name = _("Create new Field Threshold")
    icon = "plus"
    ajax = False
    classes = ("ajax-modal",)

    def get_link_url(self, datum=None):
        url = 'horizon:admin:hashmap:field_threshold_create'
        field_id = self.table.request.field_id
        return reverse(url, args=[field_id])


class DeleteServiceThreshold(tables.DeleteAction):
    name = "deletetservicethreshold"
    verbose_name = _("Delete Service Threshold")
    data_type_singular = _("Service Threshold")
    data_type_plural = _("Service Thresholds")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Delete Service Threshold",
            u"Delete Service Thresholds",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Deleted Service Threshold",
            u"Deleted Service Thresholds",
            count
        )

    def action(self, request, threshold_id):
        api.cloudkittyclient(request).rating.hashmap.delete_threshold(
            threshold_id=threshold_id)


class DeleteFieldThreshold(tables.DeleteAction):
    name = "deletefieldthreshold"
    verbose_name = _("Delete Field Threshold")
    data_type_singular = _("Field Threshold")
    data_type_plural = _("Field Thresholds")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Delete Field Threshold",
            u"Delete Field Thresholds",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Deleted Field Threshold",
            u"Deleted Field Thresholds",
            count
        )

    def action(self, request, threshold_id):
        api.cloudkittyclient(request).rating.hashmap.delete_threshold(
            threshold_id=threshold_id)


class EditServiceThreshold(tables.LinkAction):
    name = "editservicethreshold"
    verbose_name = _("Edit Service Threshold")
    icon = "edit"
    ajax = True
    classes = ("ajax-modal",)

    def get_link_url(self, datum=None):
        url = 'horizon:admin:hashmap:service_threshold_edit'
        return reverse(url, args=[datum.threshold_id])


def get_groupname(datum):
    if hasattr(datum, "group_name"):
        groupname = datum.group_name
        return groupname
    return _("Not available")


def add_groupname(request, datums):
    client = api.cloudkittyclient(request)
    groups = client.rating.hashmap.get_group().get('groups', [])
    full_groups = OrderedDict([(str(group['group_id']), group['name'])
                               for group in groups])

    for datum in datums:
        if datum.get('group_id'):
            if datum['group_id'] in full_groups:
                datum['group_name'] = full_groups[datum['group_id']]
            else:
                group = client.rating.hashmap.get_group(
                    group_id=datum['group_id'])
                datum['group_name'] = group['name']


class BaseThresholdsTable(tables.DataTable):
    level = tables.Column('level', verbose_name=_("Level"))
    type = tables.Column('type', verbose_name=_("Type"))
    cost = tables.Column('cost', verbose_name=_("Cost"))
    group_name = tables.Column(get_groupname,
                               verbose_name=_("Group Name"),
                               link=get_detail_link)
    tenant_id = tables.Column('tenant_id', verbose_name=_("Project"))


class ServiceThresholdsTable(BaseThresholdsTable):
    """This table list the available service thresholds.

    Clicking on a group name sends you to a GroupsTab page.
    """

    class Meta(object):
        name = "service_thresholds"
        verbose_name = _("Service Threshold")
        table_actions = (CreateServiceThreshold, DeleteServiceThreshold)
        row_actions = (EditServiceThreshold, DeleteServiceThreshold)


class ServiceThresholdsTab(tabs.TableTab):
    name = _("Service Thresholds")
    slug = "hashmap_service_thresholds"
    table_classes = (ServiceThresholdsTable,)
    template_name = "horizon/common/_detail_table.html"
    preload = True

    def get_service_thresholds_data(self):
        client = api.cloudkittyclient(self.request)
        thresholds = client.rating.hashmap.get_threshold(
            service_id=self.request.service_id).get('thresholds', [])
        add_groupname(self.request, thresholds)
        return api.identify(thresholds, key='threshold_id', name=True)


class EditFieldThreshold(tables.LinkAction):
    name = "editfieldthreshold"
    verbose_name = _("Edit Field Threshold")
    icon = "edit"
    ajax = True
    classes = ("ajax-modal",)

    def get_link_url(self, datum=None):
        url = 'horizon:admin:hashmap:field_threshold_edit'
        return reverse(url, args=[datum.threshold_id])


class FieldThresholdsTable(BaseThresholdsTable):
    """This table list the available field thresholds.

    Clicking on a group name sends you to a GroupsTab page.
    """

    class Meta(object):
        name = "field_thresholds"
        verbose_name = _("Field Threshold")
        table_actions = (CreateFieldThreshold, DeleteFieldThreshold)
        row_actions = (EditFieldThreshold, DeleteFieldThreshold)


class FieldThresholdsTab(tabs.TableTab):
    name = _("Field Thresholds")
    slug = "hashmap_field_thresholds"
    table_classes = (FieldThresholdsTable,)
    template_name = "horizon/common/_detail_table.html"
    preload = True

    def get_field_thresholds_data(self):
        client = api.cloudkittyclient(self.request)
        thresholds = client.rating.hashmap.get_threshold(
            field_id=self.request.field_id).get('thresholds', [])
        add_groupname(self.request, thresholds)
        return api.identify(thresholds, key='threshold_id', name=True)


class DeleteField(tables.DeleteAction):
    name = "deletefield"
    verbose_name = _("Delete Field")
    data_type_singular = _("Field")
    data_type_plural = _("Fields")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Delete Field",
            u"Delete Fields",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Deleted Field",
            u"Deleted Fields",
            count
        )

    def action(self, request, field_id):
        api.cloudkittyclient(request).rating.hashmap.delete_field(
            field_id=field_id)


class CreateField(tables.LinkAction):
    name = "createfield"
    verbose_name = _("Create new Field")
    icon = "plus"
    ajax = True
    classes = ("ajax-modal",)

    def get_link_url(self, datum=None):
        url = 'horizon:admin:hashmap:field_create'
        service_id = self.table.request.service_id
        return reverse(url, args=[service_id])


class FieldsTable(tables.DataTable):
    """This table lists the available fields for a given service.

    Clicking on a fields sends you to a MappingsTable.
    """
    name = tables.Column(
        'name',
        verbose_name=_("Name"),
        link='horizon:admin:hashmap:field')

    class Meta(object):
        name = "fields"
        verbose_name = _("Fields")
        multi_select = False
        row_actions = (DeleteField,)
        table_actions = (CreateField, DeleteField)


class FieldsTab(tabs.TableTab):
    name = _("Fields")
    slug = "hashmap_fields"
    table_classes = (FieldsTable,)
    template_name = "horizon/common/_detail_table.html"
    preload = True

    def get_fields_data(self):
        client = api.cloudkittyclient(self.request)
        fields = client.rating.hashmap.get_field(
            service_id=self.request.service_id)['fields']
        return api.identify(fields, key='field_id')


class DeleteMapping(tables.DeleteAction):
    name = "deletemapping"
    verbose_name = _("Delete Mapping")
    data_type_singular = _("Mapping")
    data_type_plural = _("Mappings")

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Delete Mapping",
            u"Delete Mappings",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Deleted Mapping",
            u"Deleted Mappings",
            count
        )

    def action(self, request, mapping_id):
        api.cloudkittyclient(request).rating.hashmap.delete_mapping(
            mapping_id=mapping_id)


class CreateServiceMapping(tables.LinkAction):
    name = "createiservicemapping"
    verbose_name = _("Create new Mapping")
    icon = "plus"
    ajax = True
    classes = ("ajax-modal",)

    def get_link_url(self, datum=None):
        url = 'horizon:admin:hashmap:service_mapping_create'
        service_id = self.table.request.service_id
        return reverse(url, args=[service_id])


class EditServiceMapping(tables.LinkAction):
    name = "editservicemapping"
    verbose_name = _("Edit Mapping")
    icon = "edit"
    ajax = True
    classes = ("ajax-modal",)

    def get_link_url(self, datum=None):
        url = 'horizon:admin:hashmap:service_mapping_edit'
        return reverse(url, args=[datum.mapping_id])


class BaseMappingsTable(tables.DataTable):
    type = tables.Column('type', verbose_name=_("Type"))
    cost = tables.Column('cost', verbose_name=_("Cost"))
    group_name = tables.Column(get_groupname,
                               verbose_name=_("Group Name"),
                               link=get_detail_link)
    tenant_id = tables.Column('tenant_id', verbose_name=_("Project"))


class ServiceMappingsTable(BaseMappingsTable):

    class Meta(object):
        name = "mappings"
        verbose_name = _("Mappings")
        row_actions = (EditServiceMapping, DeleteMapping)
        table_actions = (CreateServiceMapping, DeleteMapping)


class CreateFieldMapping(tables.LinkAction):
    name = "createfieldmapping"
    verbose_name = _("Create new Mapping")
    icon = "plus"
    ajax = True
    classes = ("ajax-modal",)

    def get_link_url(self, datum=None):
        url = 'horizon:admin:hashmap:field_mapping_create'
        field_id = self.table.request.field_id
        return reverse(url, args=[field_id])


class EditFieldMapping(tables.LinkAction):
    name = "editfieldmapping"
    verbose_name = _("Edit Mapping")
    icon = "edit"
    ajax = True
    classes = ("ajax-modal",)

    def get_link_url(self, datum=None):
        url = 'horizon:admin:hashmap:field_mapping_edit'
        return reverse(url, args=[datum.mapping_id])


class FieldMappingsTable(BaseMappingsTable):
    value = tables.Column('value', verbose_name=_("Value"))

    class Meta(object):
        name = "mappings"
        verbose_name = _("Mappings")
        row_actions = (EditFieldMapping, DeleteMapping)
        table_actions = (CreateFieldMapping, DeleteMapping)


class FieldMappingsTab(tabs.TableTab):
    name = _("Field Mappings")
    slug = "hashmap_field_mappings"
    table_classes = (FieldMappingsTable,)
    template_name = "horizon/common/_detail_table.html"
    preload = True

    def get_mappings_data(self):
        client = api.cloudkittyclient(self.request)
        mappings = client.rating.hashmap.get_mapping(
            field_id=self.request.field_id).get('mappings', [])
        add_groupname(self.request, mappings)
        return api.identify(mappings, key='mapping_id', name=True)


class MappingsTab(tabs.TableTab):
    name = _("Service Mappings")
    slug = "hashmap_mappings"
    table_classes = (ServiceMappingsTable,)
    template_name = "horizon/common/_detail_table.html"
    preload = True

    def get_mappings_data(self):
        client = api.cloudkittyclient(self.request)
        mappings = client.rating.hashmap.get_mapping(
            service_id=self.request.service_id).get('mappings', [])
        add_groupname(self.request, mappings)
        return api.identify(mappings, key='mapping_id', name=True)


class FieldTabs(tabs.TabGroup):
    slug = "field_tabs"
    tabs = (FieldMappingsTab, FieldThresholdsTab)
    sticky = True


class ServiceTabs(tabs.TabGroup):
    slug = "services_tabs"
    tabs = (FieldsTab, MappingsTab, ServiceThresholdsTab, GroupsTab)
    sticky = True



# Copyright 2015 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from horizon import forms
from horizon import tables
from horizon import tabs
from horizon import views
from keystoneauth1 import exceptions

from cloudkittydashboard.api import cloudkitty as api
from cloudkittydashboard.dashboards.admin.hashmap import forms as hashmap_forms
from cloudkittydashboard.dashboards.admin.hashmap \
    import tables as hashmap_tables


class IndexView(tables.DataTableView):
    table_class = hashmap_tables.ServicesTable
    template_name = "admin/hashmap/services_list.html"

    def get_data(self):
        manager = api.cloudkittyclient(self.request)
        services = manager.rating.hashmap.get_service().get('services', [])
        services = sorted(services, key=lambda service: service['name'])
        list_services = []
        for s in services:
            try:
                service = manager.info.get_metric(metric_name=s['name'])
                unit = service['unit']
            except exceptions.NotFound:
                unit = "-"

            list_services.append({
                "id": s['service_id'],
                "name": s['name'],
                "unit": unit
            })
        return list_services


class ServiceView(tabs.TabbedTableView):
    tab_group_class = hashmap_tables.ServiceTabs
    template_name = 'admin/hashmap/service_details.html'

    def get(self, *args, **kwargs):
        service = api.cloudkittyclient(
            self.request).rating.hashmap.get_service(
            service_id=kwargs['service_id'])
        self.request.service_id = service['service_id']
        self.page_title = "Hashmap Service : %s" % service['name']
        return super(ServiceView, self).get(*args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(ServiceView, self).get_context_data(**kwargs)
        manager = api.cloudkittyclient(self.request)
        service = manager.rating.hashmap.get_service(
            service_id=kwargs['service_id'])
        config = manager.info.get_config()
        period = None

        if service['name'] in config['metrics'].keys():
            period = config.get('period', 3600)

        context["service_period"] = period
        return context


class ServiceCreateView(forms.ModalFormView):
    form_class = hashmap_forms.CreateServiceForm
    form_id = "create_service"
    modal_header = _("Create Service")
    page_title = _("Create Service")
    success_url = reverse_lazy('horizon:admin:hashmap:index')
    submit_url = reverse_lazy('horizon:admin:hashmap:service_create')
    template_name = 'admin/hashmap/service_create.html'

    def get_object_id(self, obj):
        return obj.service_id


class FieldView(tabs.TabbedTableView):
    tab_group_class = hashmap_tables.FieldTabs
    template_name = 'admin/hashmap/field_details.html'

    def get(self, *args, **kwargs):
        field = api.cloudkittyclient(self.request).rating.hashmap.get_field(
            field_id=kwargs['field_id'])
        self.request.field_id = field['field_id']
        self.page_title = "Hashmap Field : %s" % field['name']
        return super(FieldView, self).get(*args, **kwargs)


class FieldCreateView(forms.ModalFormView):
    form_class = hashmap_forms.CreateFieldForm
    form_id = "create_field"
    modal_header = _("Create Field")
    page_title = _("Create Field")
    template_name = 'admin/hashmap/field_create.html'
    success_url = 'horizon:admin:hashmap:service'
    submit_url = 'horizon:admin:hashmap:field_create'

    def get_object_id(self, obj):
        return obj.field_id

    def get_context_data(self, **kwargs):
        context = super(FieldCreateView, self).get_context_data(**kwargs)
        context["service_id"] = self.kwargs['service_id']
        args = (self.kwargs['service_id'],)
        context['submit_url'] = reverse_lazy(self.submit_url, args=args)
        return context

    def get_initial(self):
        return {"service_id": self.kwargs["service_id"]}

    def get_success_url(self, **kwargs):
        args = (self.kwargs['service_id'],)
        return reverse_lazy(self.success_url, args=args)


class ServiceMappingCreateView(forms.ModalFormView):
    form_class = hashmap_forms.CreateServiceMappingForm
    form_id = "create_mapping"
    modal_header = _("Create Mapping")
    page_title = _("Create Mapping")
    template_name = 'admin/hashmap/mapping_create.html'
    success_url = 'horizon:admin:hashmap:service'
    submit_url = 'horizon:admin:hashmap:service_mapping_create'

    def get_object_id(self, obj):
        return obj.mapping_id

    def get_context_data(self, **kwargs):
        context = super(ServiceMappingCreateView,
                        self).get_context_data(**kwargs)
        context["service_id"] = self.kwargs.get('service_id')
        context['submit_url'] = reverse_lazy(self.submit_url,
                                             args=(context['service_id'], ))
        return context

    def get_initial(self):
        return {"service_id": self.kwargs.get("service_id")}

    def get_success_url(self, **kwargs):
        return reverse('horizon:admin:hashmap:service',
                       args=(self.kwargs['service_id'],))


class ServiceMappingEditView(ServiceMappingCreateView):
    form_class = hashmap_forms.EditServiceMappingForm
    form_id = "update_mapping"
    modal_header = _("Update Mapping")
    page_title = _("Update Mapping")
    submit_url = 'horizon:admin:hashmap:service_mapping_edit'
    success_url = 'horizon:admin:hashmap:service_mapping_edit'

    def get_initial(self):
        out = api.cloudkittyclient(self.request).rating.hashmap.get_mapping(
            mapping_id=self.kwargs['mapping_id'])
        self.initial = out
        return self.initial

    def get_context_data(self, **kwargs):
        context = super(ServiceMappingEditView,
                        self).get_context_data(**kwargs)
        context["mapping_id"] = self.kwargs.get('mapping_id')
        context['submit_url'] = reverse_lazy(self.submit_url,
                                             args=(context['mapping_id'], ))
        return context

    def get_success_url(self, **kwargs):
        return reverse('horizon:admin:hashmap:service',
                       args=(self.initial['service_id'], ))


class FieldMappingCreateView(forms.ModalFormView):
    form_class = hashmap_forms.CreateFieldMappingForm
    form_id = "create_field_mapping"
    modal_header = _("Create Field Mapping")
    page_title = _("Create field Mapping")
    template_name = 'admin/hashmap/mapping_create.html'
    submit_url = 'horizon:admin:hashmap:field_mapping_create'
    success_url = 'horizon:admin:hashmap:field_mapping_create'

    def get_object_id(self, obj):
        return obj.mapping_id

    def get_context_data(self, **kwargs):
        context = super(FieldMappingCreateView,
                        self).get_context_data(**kwargs)
        context["field_id"] = self.kwargs.get('field_id')
        context['submit_url'] = reverse_lazy(self.submit_url,
                                             args=(context['field_id'], ))
        return context

    def get_initial(self):
        return {"field_id": self.kwargs.get("field_id")}

    def get_success_url(self, **kwargs):
        return reverse('horizon:admin:hashmap:field',
                       args=(self.kwargs['field_id'], ))


class FieldMappingEditView(FieldMappingCreateView):
    form_class = hashmap_forms.EditFieldMappingForm
    form_id = "update_field_mapping"
    modal_header = _("Update Field Mapping")
    page_title = _("Update Field Mapping")
    submit_url = 'horizon:admin:hashmap:field_mapping_edit'

    def get_initial(self):
        out = api.cloudkittyclient(self.request).rating.hashmap.get_mapping(
            mapping_id=self.kwargs['mapping_id'])
        self.initial = out
        return self.initial

    def get_context_data(self, **kwargs):
        context = super(FieldMappingEditView,
                        self).get_context_data(**kwargs)
        context["mapping_id"] = self.kwargs.get('mapping_id')
        context['submit_url'] = reverse_lazy(self.submit_url,
                                             args=(context['mapping_id'], ))
        return context

    def get_success_url(self, **kwargs):
        return reverse('horizon:admin:hashmap:field',
                       args=(self.initial['field_id'], ))


class GroupCreateView(forms.ModalFormView):
    form_class = hashmap_forms.CreateGroupForm
    form_id = "create_group"
    modal_header = _("Create Group")
    page_title = _("Create Group")
    template_name = 'admin/hashmap/group_create.html'
    submit_url = 'horizon:admin:hashmap:group_create'
    success_url = 'horizon:admin:hashmap:group_create'

    def get_success_url(self, **kwargs):
        return reverse('horizon:admin:hashmap:service',
                       args=(self.kwargs['service_id'],))

    def get_object_id(self, obj):
        return obj.group_id

    def get_context_data(self, **kwargs):
        context = super(GroupCreateView,
                        self).get_context_data(**kwargs)
        context["service_id"] = self.kwargs.get('service_id')
        context['submit_url'] = reverse_lazy(self.submit_url,
                                             args=(context['service_id'], ))
        return context

    '''
    def get_success_url(self, **kwargs):
        return reverse('horizon:admin:hashmap:group',
                       args=(kwargs['group_id'], ))
    '''


class ServiceThresholdCreateView(forms.ModalFormView):
    form_class = hashmap_forms.CreateServiceThresholdForm
    form_id = "create_service_threshold"
    modal_header = _("Create Service Threshold")
    page_title = _("Create Service Threshold")
    template_name = 'admin/hashmap/threshold_create.html'
    success_url = 'horizon:admin:hashmap:service'
    submit_url = 'horizon:admin:hashmap:service_threshold_create'

    def get_object_id(self, obj):
        return obj.field_id

    def get_success_url(self, **kwargs):
        return reverse('horizon:admin:hashmap:service',
                       args=(self.kwargs['service_id'],))

    def get_context_data(self, **kwargs):
        context = super(ServiceThresholdCreateView,
                        self).get_context_data(**kwargs)
        context["service_id"] = self.kwargs.get('service_id')
        args = (context['service_id'],)
        context['submit_url'] = reverse_lazy(self.submit_url, args=args)
        return context

    def get_initial(self):
        return {"service_id": self.kwargs["service_id"]}


class ServiceThresholdEditView(ServiceThresholdCreateView):
    form_class = hashmap_forms.EditServiceThresholdForm
    form_id = "update_service_threshold"
    modal_header = _("Update Service Threshold")
    page_title = _("Update Service Threshold")
    submit_url = 'horizon:admin:hashmap:service_threshold_edit'

    def get_initial(self):
        out = api.cloudkittyclient(self.request).rating.hashmap.get_threshold(
            threshold_id=self.kwargs['threshold_id'])
        self.initial = out
        return self.initial

    def get_context_data(self, **kwargs):
        context = super(ServiceThresholdEditView,
                        self).get_context_data(**kwargs)
        context["threshold_id"] = self.kwargs.get('threshold_id')
        context['submit_url'] = reverse_lazy(self.submit_url,
                                             args=(context['threshold_id'], ))
        return context

    def get_success_url(self, **kwargs):
        return reverse('horizon:admin:hashmap:service',
                       args=(self.initial['service_id'], ))


class FieldThresholdCreateView(forms.ModalFormView):
    form_class = hashmap_forms.CreateFieldThresholdForm
    form_id = "create_field_threshold"
    modal_header = _("Create Field Threshold")
    page_title = _("Create Field Threshold")
    template_name = 'admin/hashmap/threshold_create.html'
    success_url = 'horizon:admin:hashmap:field'
    submit_url = 'horizon:admin:hashmap:field_threshold_create'

    def get_object_id(self, obj):
        return obj.field_id

    def get_success_url(self, **kwargs):
        return reverse('horizon:admin:hashmap:field',
                       args=(self.kwargs['field_id'],))

    def get_context_data(self, **kwargs):
        context = super(FieldThresholdCreateView,
                        self).get_context_data(**kwargs)
        context["field_id"] = self.kwargs.get('field_id')
        args = (context['field_id'],)
        context['submit_url'] = reverse_lazy(self.submit_url, args=args)
        return context

    def get_initial(self):
        return {"field_id": self.kwargs["field_id"]}


class FieldThresholdEditView(FieldThresholdCreateView):
    form_class = hashmap_forms.EditFieldThresholdForm
    form_id = "update_field_threshold"
    modal_header = _("Update Field Threshold")
    page_title = _("Update Field Threshold")
    submit_url = 'horizon:admin:hashmap:field_threshold_edit'

    def get_initial(self):
        out = api.cloudkittyclient(self.request).rating.hashmap.get_threshold(
            threshold_id=self.kwargs['threshold_id'])
        self.initial = out
        return self.initial

    def get_context_data(self, **kwargs):
        context = super(FieldThresholdEditView,
                        self).get_context_data(**kwargs)
        context["threshold_id"] = self.kwargs.get('threshold_id')
        context['submit_url'] = reverse_lazy(self.submit_url,
                                             args=(context['threshold_id'], ))
        return context

    def get_success_url(self, **kwargs):
        return reverse('horizon:admin:hashmap:field',
                       args=(self.initial['field_id'], ))


class GroupView(tabs.TabbedTableView):
    tab_group_class = hashmap_tables.GroupsTab
    template_name = 'admin/hashmap/group_details.html'

    def get(self, *args, **kwargs):
        group = api.cloudkittyclient(self.request).rating.hashmap.get_group(
            group_id=kwargs['group_id']
        )
        self.request.group_id = group.group_id
        self.page_title = "Hashmap Group : %s" % group.name
        return super(GroupView, self).get(*args, **kwargs)

    def get_data(self):
        out = api.cloudkittyclient(self.request).rating.hashmap.get_group()
        return api.identify(out)


class GroupDetailsView(views.APIView):
    template_name = 'admin/hashmap/group_details.html'
    page_title = _("Group Details")

    def get_data(self, request, context, *args, **kwargs):
        group_id = kwargs.get("group_id")
        ck_client = api.cloudkittyclient(self.request)

        try:
            group = ck_client.rating.hashmap.get_group(group_id=group_id)
        except Exception:
            group = None
        try:
            mappings = ck_client.rating.hashmap.get_mapping(
                group_id=group_id)['mappings']
        except Exception:
            mappings = []
        try:
            thresholds = ck_client.rating.hashmap.get_threshold(
                group_id=group_id)['thresholds']
        except Exception:
            thresholds = []

        values = {
            "mappings": {"fields": [], "services": []},
            "thresholds": {"fields": [], "services": []}
            }
        for key, value in dict(
                mappings=mappings, thresholds=thresholds).items():
            for entry in value:
                if entry.get('service_id'):
                    values[key]['services'].append(entry)
                else:
                    values[key]['fields'].append(entry)
        context.update(values)
        context['group'] = group
        return context



# Copyright 2015 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import json

from django.conf import settings
from django import http
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import tables

from cloudkittydashboard.api import cloudkitty as api
from cloudkittydashboard.dashboards.project.rating \
    import tables as rating_tables


class IndexView(tables.DataTableView):
    table_class = rating_tables.SummaryTable
    template_name = 'project/rating/index.html'

    def get_data(self):
        summary = api.cloudkittyclient(
            self.request, version='2').summary.get_summary(
                tenant_id=self.request.user.tenant_id,
                groupby=['type'], response_format='object')

        data = summary.get('results')
        total = sum([r.get('rate') for r in data])

        data.append({'type': 'TOTAL', 'rate': total})
        return data


def quote(request):
    pricing = 0.0
    if request.is_ajax():
        if request.method == 'POST':
            json_data = json.loads(request.body)

            def __update_quotation_data(element, service):
                if isinstance(element, dict):
                    element['service'] = service
                else:
                    for elem in element:
                        __update_quotation_data(elem, service)

            try:
                service = getattr(
                    settings, 'CLOUDKITTY_QUOTATION_SERVICE', 'instance')
                __update_quotation_data(json_data, service)
                pricing = float(api.cloudkittyclient(request)
                                .rating.get_quotation(res_data=json_data))
            except Exception:
                exceptions.handle(request,
                                  _('Unable to retrieve price.'))

    return http.HttpResponse(json.dumps(pricing),
                             content_type='application/json')



# Copyright 2015 Objectif Libre
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django.utils.text import normalize_newlines
from django.utils.translation import gettext_lazy as _
from horizon import exceptions
from horizon import forms
from horizon import messages

from cloudkittydashboard.api import cloudkitty as api

LOG = logging.getLogger(__name__)


class CreateScriptForm(forms.SelfHandlingForm):
    help_text = _('Create a new rating script.')
    name = forms.CharField(label=_("Name"))
    source_choices = [('raw', _('Direct Input')),
                      ('file', _('File'))]
    script_source = forms.ChoiceField(
        label=_('Rating Script Source'),
        choices=source_choices,
        widget=forms.Select(attrs={
            'class': 'switchable',
            'data-slug': 'scriptsource'}))
    script_help = _("A script or set of python commands to modify rating "
                    "calculations.")
    script_upload = forms.FileField(
        label=_('Script File'),
        help_text=script_help,
        widget=forms.FileInput(attrs={
            'class': 'switched',
            'data-required-when-shown': 'true',
            'data-switch-on': 'scriptsource',
            'data-scriptsource-file': _('Script File')}),
        required=False)
    script_data = forms.CharField(
        label=_('Script Data'),
        help_text=script_help,
        widget=forms.widgets.Textarea(attrs={
            'class': 'switched',
            'data-required-when-shown': 'true',
            'data-switch-on': 'scriptsource',
            'data-scriptsource-raw': _('Script Data')}),
        required=False)

    class Meta(object):
        name = _('Create Script')

    def clean(self):
        cleaned = super(CreateScriptForm, self).clean()

        files = self.request.FILES
        script = self.clean_uploaded_files('script', files)

        if script is not None:
            cleaned['script_data'] = script

        return cleaned

    def clean_uploaded_files(self, prefix, files):
        upload_str = prefix + "_upload"

        has_upload = upload_str in files
        if has_upload:
            upload_file = files[upload_str]
            log_script_name = upload_file.name
            LOG.info('got upload %s' % log_script_name)
            script = upload_file.read()
            if script != "":
                try:
                    normalize_newlines(script)
                except Exception as e:
                    msg = _('There was a problem parsing the'
                            ' %(prefix)s: %(error)s')
                    msg = msg % {'prefix': prefix, 'error': e}
                    raise forms.ValidationError(msg)
            return script
        else:
            return None

    def handle(self, request, data):
        name = data['name']
        LOG.info('Creating script with name %s' % (name))
        ck_client = api.cloudkittyclient(request)
        try:
            script = ck_client.rating.pyscripts.create_script(
                name=name,
                data=data['script_data'])
            messages.success(
                request,
                _('Successfully created script'))
            return script
        except Exception:
            exceptions.handle(request,
                              _("Unable to create script."))


class EditScriptForm(CreateScriptForm):
    script_id = forms.CharField(label=_("Script ID"),
                                widget=forms.TextInput(
                                attrs={'readonly': 'readonly'}))
    fields_order = ['script_id', 'name', 'script_source', 'script_upload',
                    'script_data']

    class Meta(object):
        name = _("Update Script")

    def handle(self, request, data):
        script_id = self.initial['script_id']
        LOG.info('Updating script with id %s' % (script_id))
        ck_client = api.cloudkittyclient(request)
        try:
            script = ck_client.rating.pyscripts.update_script(
                script_id=script_id, name=data['name'],
                data=data['script_data'])
            messages.success(
                request,
                _('Successfully updated script'))
            return script
        except Exception:
            exceptions.handle(request,
                              _("Unable to update script."))
